package org.caosdb.server.database.backend.implementation.MySQL;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;
import org.caosdb.server.database.access.Access;
import org.caosdb.server.database.exceptions.TransactionException;
import org.caosdb.server.database.proto.SparseEntity;
import org.caosdb.server.entity.EntityID;

public class MySQLGetFilesInDirectory extends MySQLTransaction
    implements org.caosdb.server.database.backend.interfaces.GetFilesInDirectoryImpl {

  public MySQLGetFilesInDirectory(final Access access) {
    super(access);
  }

  public static final String STMT_GET_FILES_IN_DIR = "call listFSODescriptorByParentDirectory(?)";

  @Override
  public List<SparseEntity> execute(final EntityID directory_id) {
    try {
      final PreparedStatement stmt = prepareStatement(STMT_GET_FILES_IN_DIR);
      stmt.setString(1, directory_id.toString());
      final List<SparseEntity> result = new LinkedList<>();
      try (ResultSet rs = stmt.executeQuery()) {
        while (rs.next()) {
          final SparseEntity entity = new SparseEntity();
          entity.id = rs.getString("FileId");
          DatabaseUtils.parseFSODescriptorFields(rs, entity);
          result.add(entity);
        }
      }
      return result;
    } catch (SQLException | ConnectionException e) {
      throw new TransactionException(e);
    }
  }
}
