package org.caosdb.server.database.backend.implementation.MySQL;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.caosdb.server.database.access.Access;
import org.caosdb.server.database.backend.interfaces.GetVirtualFSOImpl;
import org.caosdb.server.database.exceptions.TransactionException;
import org.caosdb.server.database.proto.SparseEntity;
import org.caosdb.server.filesystem.FSODescriptor;
import org.caosdb.server.filesystem.VirtualFSODescriptorInterface;

public class MySQLGetVirtualFSO extends MySQLTransaction implements GetVirtualFSOImpl {

  public MySQLGetVirtualFSO(Access access) {
    super(access);
  }

  public static final String STMT_GET_FSO = "call retrieveFSODescriptor(?, ?)";

  @Override
  public VirtualFSODescriptorInterface execute(String fileStorageId, String key) {
    try {
      PreparedStatement stmt = prepareStatement(STMT_GET_FSO);
      stmt.setString(1, fileStorageId);
      stmt.setString(2, key);
      ResultSet resultSet = stmt.executeQuery();
      if (resultSet.next()) {
        SparseEntity e = new SparseEntity();
        e.id = resultSet.getString("FileId");
        DatabaseUtils.parseFSODescriptorFields(resultSet, e);
        return new FSODescriptor(e);
      }
    } catch (SQLException | ConnectionException e) {
      throw new TransactionException(e);
    }
    return null;
  }
}
