package org.caosdb.server.database.backend.implementation.MySQL;

import java.sql.PreparedStatement;
import java.sql.SQLIntegrityConstraintViolationException;
import java.sql.Types;
import org.caosdb.server.database.access.Access;
import org.caosdb.server.database.backend.interfaces.InsertFSODescriptorImpl;
import org.caosdb.server.database.exceptions.IntegrityException;
import org.caosdb.server.database.exceptions.TransactionException;
import org.caosdb.server.database.proto.SparseEntity;
import org.caosdb.server.filesystem.FileSystem.ObjectType;

public class MySQLInsertFSODescriptor extends MySQLTransaction implements InsertFSODescriptorImpl {

  public MySQLInsertFSODescriptor(final Access access) {
    super(access);
  }

  public static final String STMT_INSERT_FILE_PROPERTIES =
      "call insertFSODescriptor(?,?,?, ?,?,?, ?,?,?, ?,?)";

  @Override
  public void insertFSODescriptor(final SparseEntity entity) {
    try {
      final PreparedStatement insertFilePropsStmt = prepareStatement(STMT_INSERT_FILE_PROPERTIES);
      if (entity.filePath != null) {
        insertFilePropsStmt.setString(1, entity.id);
        if (entity.fileHash != null) {
          insertFilePropsStmt.setString(2, entity.fileHash.value);
          insertFilePropsStmt.setString(3, entity.fileHash.algorithm);
          insertFilePropsStmt.setLong(4, entity.fileHash.datetime);
        } else {
          insertFilePropsStmt.setNull(2, Types.BINARY);
          insertFilePropsStmt.setNull(3, Types.VARCHAR);
          insertFilePropsStmt.setLong(4, 0);
        }
        if (entity.fileSize != null) {
          insertFilePropsStmt.setLong(5, entity.fileSize);
        } else {
          insertFilePropsStmt.setNull(5, Types.BIGINT);
        }
        insertFilePropsStmt.setString(6, entity.filePath.toString());
        if (entity.fileParentId != null) {
          insertFilePropsStmt.setString(7, entity.fileParentId);
        } else {
          insertFilePropsStmt.setNull(7, Types.VARCHAR);
        }
        if (entity.fileType == ObjectType.DIRECTORY) {
          insertFilePropsStmt.setString(8, "inode/directory");
        } else if (entity.fileType == ObjectType.LINK) {
          insertFilePropsStmt.setString(8, "inode/link");
        } else if (entity.fileMimeType != null) {
          insertFilePropsStmt.setString(8, entity.fileMimeType);
        } else {
          insertFilePropsStmt.setNull(8, Types.VARBINARY);
        }
        insertFilePropsStmt.setString(9, entity.fileStorageId);
        insertFilePropsStmt.setString(10, entity.fileKey);
        if (entity.linkTarget != null) {
          insertFilePropsStmt.setString(11, entity.linkTarget);
        } else {
          insertFilePropsStmt.setNull(11, Types.VARCHAR);
        }
        insertFilePropsStmt.execute();
      }
    } catch (final SQLIntegrityConstraintViolationException exc) {
      throw new IntegrityException(exc);
    } catch (final TransactionException exc) {
      throw exc;
    } catch (final Exception exc) {
      throw new TransactionException(exc);
    }
  }
}
