/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package org.caosdb.server.database.backend.interfaces;

import org.caosdb.server.database.exceptions.TransactionException;
import org.caosdb.server.database.proto.SparseEntity;
import org.caosdb.server.filesystem.Path;

/**
 * Resolves a path in the virtual directory structure of the CaosDB file system and returns a {@link
 * SparseEntity} which represents the corresponding file entity.
 *
 * @author Timm Fitschen (t.fitschen@indiscale.com)
 */
public interface GetFileEntityByPathImpl extends BackendTransactionImpl {

  /**
   * Return the SparseEntity which belongs to the given path.
   *
   * <p>TODO refactor and remove the `like` option.
   *
   * @param path
   * @param like
   * @return
   * @throws TransactionException
   */
  public abstract SparseEntity execute(Path path, boolean like) throws TransactionException;
}
