/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package org.caosdb.server.database.backend.transaction;

import java.util.Iterator;
import org.caosdb.server.database.BackendTransaction;
import org.caosdb.server.database.backend.interfaces.ListFilesImpl;
import org.caosdb.server.database.exceptions.TransactionException;
import org.caosdb.server.filesystem.Path;
import org.caosdb.server.filesystem.VirtualFSODescriptorInterface;

/**
 * Generate an Iterator over all file paths in a file storage (below a particular directory).
 *
 * @author Timm Fitschen (t.fitschen@indiscale.com)
 */
public class GetFileIterator extends BackendTransaction {

  public static class FSOIterator implements Iterator<VirtualFSODescriptorInterface> {

    private GetFileIterator getFileIterator;

    public FSOIterator(GetFileIterator getFileIterator) {
      this.getFileIterator = getFileIterator;
    }

    @Override
    public boolean hasNext() {
      if (this.getFileIterator.chunk == null || !this.getFileIterator.chunk.hasNext()) {
        this.getFileIterator.execute();
      }
      return this.getFileIterator.chunk.hasNext();
    }

    @Override
    public VirtualFSODescriptorInterface next() {
      return this.getFileIterator.chunk.next();
    }
  }

  private Iterator<VirtualFSODescriptorInterface> chunk = null;
  private Iterator<VirtualFSODescriptorInterface> iterator = new FSOIterator(this);
  private String fileStorageId;
  private Path belowPath;
  private Long lastCheckedTimestamp;

  public GetFileIterator(String fileStorageId, Path belowPath, Long lastCheckedTimestamp) {
    this.fileStorageId = fileStorageId;
    this.belowPath = belowPath;
    this.lastCheckedTimestamp = lastCheckedTimestamp;
  }

  @Override
  protected void execute() throws TransactionException {
    ListFilesImpl t = getImplementation(ListFilesImpl.class);
    this.chunk = t.execute(fileStorageId, belowPath, lastCheckedTimestamp, 100).iterator();
  }

  public Iterator<VirtualFSODescriptorInterface> getIterator() {
    return this.iterator;
  }
}
