/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package org.caosdb.server.entity.xml;

import org.caosdb.server.entity.EntityInterface;
import org.caosdb.server.filesystem.FileSystem.ObjectType;
import org.jdom2.Attribute;
import org.jdom2.Element;

public class FileToElementStrategy extends EntityToElementStrategy {

  public FileToElementStrategy(String tag) {
    super(tag);
  }

  @Override
  public Element toElement(
      final EntityInterface entity, final SerializeFieldStrategy serializeFieldStrategy) {
    final Element element = super.toElement(entity, serializeFieldStrategy);

    if (entity.hasFSODescriptor()) {
      if (serializeFieldStrategy.isToBeSet("checksum") && entity.getFSODescriptor().hasHash()) {
        element.setAttribute(new Attribute("checksum", entity.getFSODescriptor().getHash().value));
      }
      if (serializeFieldStrategy.isToBeSet("path") && entity.getFSODescriptor().hasPath()) {
        element.setAttribute(new Attribute("path", "/" + entity.getFSODescriptor().getPath()));
      }
      if (serializeFieldStrategy.isToBeSet("size") && entity.getFSODescriptor().hasSize()) {
        element.setAttribute(
            new Attribute("size", Long.toString(entity.getFSODescriptor().getSize())));
      }
      if (serializeFieldStrategy.isToBeSet("linktarget")
          && entity.getFSODescriptor().getObjectType() == ObjectType.LINK
          && entity.getFSODescriptor().getLinkTarget() != null) {
        element.setAttribute("linktarget", entity.getFSODescriptor().getLinkTarget().toString());
      }
    }
    return element;
  }
}
