package org.caosdb.server.filesystem;

import java.nio.file.Path;
import java.util.LinkedList;
import java.util.List;
import org.caosdb.server.entity.Message;
import org.caosdb.server.utils.Undoable;

/**
 * This is the canonical implementation of the {@link FileSystemInterface}.
 *
 * <p>It handles all access to files (read and write) in the CaosDB server.
 *
 * @author Timm Fitschen (t.fitschen@indiscale.com)
 */
public class FileSystem implements FileSystemInterface {

  public enum ObjectType {
    FILE,
    DIRECTORY,
    LINK,
  }

  private static final FileSystem instance = new FileSystem();
  public static final String DEFAULT_BACKEND = "DEFAULT";
  private final FileStorageInterface defaultFileStorage = new DefaultLocalFileStorage();

  public FileStorageInterface getFileStorage(final FSODescriptorInterface file) {
    return getFileStorage(file.getFileStorageId());
  }

  public FileStorageInterface getFileStorage(final String id) {
    // TODO
    return defaultFileStorage;
  }

  @Override
  public RealFSODescriptorInterface resolve(final String fileStorageId, final String fileKey) {
    final FileStorageInterface fs = getFileStorage(fileStorageId);
    return fs.resolve(fileKey);
  }

  @Override
  public Undoable delete(final FSODescriptorInterface file) throws Message {
    final FileStorageInterface fs = getFileStorage(file);
    return fs.delete(file.getKey());
  }

  public static FileSystem getInstance() {
    return instance;
  }

  @Deprecated
  public List<Path> getLocalFilesStorageRoots() {
    final List<Path> result = new LinkedList<>();

    // TODO
    if (defaultFileStorage instanceof LocalFileStorage) {
      result.add(((LocalFileStorage) defaultFileStorage).getRoot());
    }
    return result;
  }

  public String[] getFileStorages() {
    final String[] result = {DEFAULT_BACKEND};
    return result;
  }

  //  public static FSODescriptorInterface resolveLinkTarget(FSODescriptorInterface fso) {
  //    return resolveLinkTarget(fso.getFileStorageId(), fso.getKey());
  //  }

  //  private static FSODescriptorInterface resolveLinkTarget(String fileStorageId, String key) {
  //    FileStorageInterface fileStorage = getInstance().getFileStorage(fileStorageId);
  //    return fileStorage.resolveLinkTarget(key);
  //  }

  //  public static boolean resolveKey(FSODescriptorInterface fso) {
  //    for (String fs : getInstance().getFileStorages()) {
  //      if (getInstance().getFileStorage(fs).resolveKey(fso)) {
  //        return true;
  //      }
  //    }
  //    return false;
  //  }

  public static Boolean exists(FSODescriptorInterface fso) {
    FileStorageInterface fileStorage = getInstance().getFileStorage(fso);
    if (fileStorage.getCapabilities().existence) {
      return fileStorage.exists(fso.getKey());
    }
    return null;
  }

  public Undoable move(RealFSODescriptorInterface fso, VirtualFSODescriptorInterface target)
      throws Message {
    FileStorageInterface fileStorage = getFileStorage(target);
    return fileStorage.move(fso, target);
  }

  //  public static boolean resolve(FSODescriptorInterface fso) {
  //    FileStorageInterface fileStorage = getInstance().getFileStorage(fso);
  //    return fileStorage.resolve(fso);
  //  }
}
