package org.caosdb.server.filesystem;

import java.io.Serializable;
import java.util.Arrays;

public class Path implements Serializable {

  private static final long serialVersionUID = -5026988213037397956L;

  public static final String DEFAULT_PATH_SEPARATOR = "/";
  private final String[] segments;

  public Path(final String path) {
    this(path, DEFAULT_PATH_SEPARATOR);
  }

  public Path(final String path, final String sep) {
    this.segments = path.replaceFirst("^" + sep, "").replaceFirst(sep + "$", "").split(sep);
  }

  private Path(final String[] segments) {
    this.segments = segments;
  }

  @Override
  public String toString() {
    return toString(DEFAULT_PATH_SEPARATOR);
  }

  public String toString(final String sep) {
    return String.join(sep, segments);
  }

  public String getLastSegment() {
    if (segments.length > 0) {
      return segments[segments.length - 1];
    }
    return null;
  }

  public String[] getSegments() {
    return segments;
  }

  public boolean isEmpty() {
    return segments.length == 0 || segments.length == 1 && segments[0].isBlank();
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj instanceof Path) {
      final Path that = (Path) obj;
      return Arrays.equals(this.segments, that.segments);
    }
    return false;
  }

  @Override
  public int hashCode() {
    return Arrays.hashCode(this.segments);
  }

  public Path getParent() {
    if (segments.length > 1) {
      return new Path(Arrays.copyOfRange(this.segments, 0, segments.length - 1));
    }
    return null;
  }

  public Path append(String name) {
    if (segments.length > 0 && !(segments.length == 1 && segments[0].isBlank())) {
      String[] newSegments = new String[this.segments.length + 1];
      for (int i = 0; i < segments.length; i++) {
        newSegments[i] = this.segments[i];
      }
      newSegments[segments.length] = name;
      return new Path(newSegments);
    }
    return new Path(name);
  }
}
