package org.caosdb.server.filesystem;

import java.util.List;
import org.caosdb.server.entity.EntityID;
import org.caosdb.server.utils.EntityStatus;

public interface VirtualFSODescriptorInterface extends FSODescriptorInterface {

  /**
   * Return the path in the virtual internal file system or null.
   *
   * <p>This path might and and will often be related to a path on another file system (e.g. a POSIX
   * one), but this is not necessarily the case (e.g. when the actual blob is stored in an * Amazon
   * S3).
   *
   * @return path in the internal file system.
   */
  public abstract Path getPath();

  /**
   * Return true iff this file has a path.
   *
   * <p>Files do not necessarily have a path. In this case they are accessible via their entity id
   * but not via the browsing the virtual file system.
   */
  public abstract boolean hasPath();

  /**
   * Set the hash.
   *
   * @param hash the new hash.
   */
  public abstract void setHash(Hash hash);

  /**
   * Set the file's byte size.
   *
   * @param size the new size.
   */
  public abstract void setSize(Long size);

  /**
   * Set the path on this file in the virtual internal file system.
   *
   * @param path the new path.
   */
  public abstract void setPath(Path path);

  // TODO
  public abstract EntityStatus getEntityStatus();

  // TODO
  public abstract void setEntityStatus(EntityStatus status);

  /** Return the id of the owner of this file system object. */
  public abstract EntityID getEntityId();

  /**
   * Set the mime type of this file.
   *
   * @param fileMimeType the new mime type
   */
  public abstract void setMimeType(String fileMimeType);

  public abstract void setChildren(List<VirtualFSODescriptorInterface> children);

  /**
   * Update this object's hash's timestamp.
   *
   * @param hash
   */
  public abstract void updateHash(Hash hash);

  public abstract EntityID getParentDirectoryID();

  public abstract void setParentDirectory(EntityID id);

  public abstract void setParentDirectory(VirtualFSODescriptorInterface newFD);

  public abstract EntityID getLinkTarget();

  /**
   * Return true iff this file has a size.
   *
   * <p>False means that the size is unknown or that this FSO does not have a defined size (e.g.
   * when this is directory).
   */
  public abstract boolean hasSize();

  /**
   * Return true iff this file has a hash.
   *
   * <p>False means that the hash is unknown or that this FSO does not have a defined hash (e.g.
   * when this is directory).
   */
  public abstract boolean hasHash();

  /**
   * Return the tmpIdentifier.
   *
   * <p>The tmpIdentifier is used to identify {@link FSODescriptorInterface} instances which are
   * members of an entity with the uploaded files in the {@link TemporaryStorage} during a
   * transaction.
   */
  public abstract String getTmpIdentifier();

  /**
   * Set the tmpIdentifier.
   *
   * @param tmpIdentifier the new tmpIdentifier.
   */
  public abstract void setTmpIdentifier(String tmpIdentifier);

  /** Return true iff this object has a tmpIdentifier. */
  public abstract boolean hasTmpIdentifier();

  public abstract RealFSODescriptorInterface getTwin();

  public abstract void setTwin(RealFSODescriptorInterface twin);

  @Override
  public abstract List<? extends VirtualFSODescriptorInterface> listChildren();

  public abstract void setKey(String key);

  public abstract Path getLinkTargetPath();

  public abstract void setLinkTargetPath(Path target);
}
