package org.caosdb.server.filesystem.consistency;

import com.fasterxml.jackson.annotation.JsonAnySetter;
import java.util.HashMap;
import java.util.Map;

public class ConsistencyConfig {

  public enum LogLevel {
    IGNORE,
    ERROR,
    WARN,
    INFO,
  }

  public static class EventsMapping<T> {
    private Map<String, T> mapping = new HashMap<>();
    protected T defaul;

    @JsonAnySetter
    public void set(String key, T val) {
      this.mapping.put(key, val);
    }

    public T get(String event) {
      return mapping.getOrDefault(event, defaul);
    }
  }

  /**
   * The EventsLogConfig maps {@link ConsistencyEventInterface}s to {@link LogLevel}s.
   *
   * <p>When the consistency check of a file storage fires an event this mapping determines the
   * severity of it.
   *
   * @author tf
   */
  public static class EventsLogConfig extends EventsMapping<LogLevel> {
    public EventsLogConfig() {
      this.defaul = LogLevel.ERROR;
      this.set(ConsistencyEvent.UNKNOWN, LogLevel.WARN);
    }
  }

  /**
   * The AutoUpdateConfig maps {@link ConsistencyEventInterface}s to {@link AutoUpdateAction}s.
   *
   * <p>When the consistency check of a file storage fires an event this mapping determines whether
   * and how the server attempts to react to the event automatically. E.g. by inserting a newly
   * found file automatically.
   *
   * @author tf
   */
  public static class AutoUpdateConfig extends EventsMapping<AutoUpdateAction> {}

  public EventsLogConfig eventsLogConfig = new EventsLogConfig();
  public AutoUpdateConfig autoUpdateConfig = new AutoUpdateConfig();
  private static ConsistencyConfig instance = new ConsistencyConfig();

  public static ConsistencyConfig getInstance() {
    return instance;
  }
}
