package org.caosdb.server.filesystem.consistency;

public interface ConsistencyEventInterface {

  /**
   * Return a universal unique identifier for this event. This can be used by other events or by
   * triggered actions for referencing this event.
   */
  public String getUUID();

  /**
   * If applicable, return the id of the file storage which this event applies to.
   *
   * <p>Otherwise, return null.
   */
  public String getFileStorageId();

  /**
   * If applicable, return the key of the file which this event applies to.
   *
   * <p>Otherwise, return null.
   *
   * <p>If this is not null, {@link #getFileStorageId()} must not be null.
   */
  public String getKey();

  /** Return the type of this event. */
  public String getEventType();

  /** Return true iff this event is a subtype of the given other event type. */
  public boolean isSubtypeOf(String eventType);

  /** Return the unix time stamp when this event was initialized. */
  Long getTimestamp();
}
