package org.caosdb.server.grpc;

import com.google.protobuf.ByteString;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import org.caosdb.api.entity.v1.FileChunk;
import org.caosdb.api.entity.v1.FileTransmissionSettings;
import org.caosdb.api.entity.v1.FileTransmissionSettings.Builder;
import org.caosdb.api.entity.v1.TransmissionStatus;
import org.caosdb.server.filesystem.RealFSODescriptorInterface;
import org.caosdb.server.filesystem.TemporaryStorage;

public class FileUpload extends FileTransmission {

  Map<String, UploadBuffer> buffers = new HashMap<>();

  public FileUpload(final String id) {
    super(id);
  }

  @Override
  public void cleanUp() {
    for (UploadBuffer buf : buffers.values()) {
      buf.cleanUp();
    }
  }

  public TransmissionStatus upload(final String fileId, final ByteString data) throws IOException {
    synchronized (lock) {
      touch();
      return getFileBuffer(fileId).write(data);
    }
  }

  protected UploadBuffer getFileBuffer(final String fileId) {
    touch();
    if (!buffers.containsKey(fileId)) {
      buffers.put(
          fileId,
          new UploadBuffer(
              TemporaryStorage.getInstance().createUploadFSODescriptorFactory(id, fileId)));
    }
    return buffers.get(fileId);
  }

  @Override
  public RealFSODescriptorInterface getFile(final String fileId) {
    synchronized (lock) {
      touch();
      if (buffers.containsKey(fileId)) {
        return buffers.get(fileId).toFileDescriptor();
      }
      return null;
    }
  }

  @Override
  public FileTransmissionSettings getTransmissionSettings() {
    final Builder builder = FileTransmissionSettings.newBuilder();
    builder.setMaxChunkSize(getMaxChunkSize());
    builder.setMaxFileSize(getMaxFileSize());
    return builder.build();
  }

  public TransmissionStatus uploadChunk(final String fileId, final FileChunk chunk)
      throws IOException {
    return upload(fileId, chunk.getData());
  }
}
