package org.caosdb.server.grpc;

import com.google.protobuf.ByteString;
import java.io.IOException;
import java.io.OutputStream;
import org.caosdb.api.entity.v1.TransmissionStatus;
import org.caosdb.server.filesystem.RealFSODescriptorInterface;
import org.caosdb.server.filesystem.TemporaryFSODescriptor;
import org.caosdb.server.filesystem.TemporaryStorage.UploadFSODescriptorFactory;

public class UploadBuffer {

  TransmissionStatus status = TransmissionStatus.TRANSMISSION_STATUS_UNSPECIFIED;
  private final UploadFSODescriptorFactory tmpFile;

  public UploadBuffer(final UploadFSODescriptorFactory tmpFile) {
    this.tmpFile = tmpFile;
  }

  public TransmissionStatus write(final ByteString data) throws IOException {
    switch (status) {
      case TRANSMISSION_STATUS_UNSPECIFIED:
        status = TransmissionStatus.TRANSMISSION_STATUS_GO_ON;
        break;
      case TRANSMISSION_STATUS_GO_ON:
        break;
      default:
        throw new RuntimeException("Wrong transmission state.");
    }
    final OutputStream outputStream = tmpFile.getOutputStream();
    data.writeTo(outputStream);
    return status;
  }

  public RealFSODescriptorInterface toFileDescriptor() {
    switch (status) {
      case TRANSMISSION_STATUS_SUCCESS:
        break;
      case TRANSMISSION_STATUS_GO_ON:
        status = TransmissionStatus.TRANSMISSION_STATUS_SUCCESS;
        break;
      default:
        throw new RuntimeException("Wrong transmission state.");
    }
    TemporaryFSODescriptor result = tmpFile.create();
    return result;
  }

  public void cleanUp() {
    this.tmpFile.cleanUp();
  }
}
