/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package org.caosdb.server.jobs;

/**
 * EntityFlagJob
 *
 * <p>An entity job with also passes a flag to the implementing class.
 */
public abstract class EntityFlagJob extends EntityJob {

  protected String value = null;

  public void setValue(final String value) {
    this.value = value;
  }

  @Override
  protected void run() {
    if (this.value == null) {
      JobAnnotation annotation = getClass().getAnnotation(JobAnnotation.class);
      if (getContainer().getFlags() != null) {
        this.value =
            this.getContainer()
                .getFlags()
                .getOrDefault(annotation.flag(), annotation.defaultValue());
      } else {
        this.value = annotation.defaultValue();
      }
    }
    job(this.value);
  }

  protected abstract void job(final String value);
}
