/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package org.caosdb.server.jobs.core;

import org.caosdb.server.database.backend.transaction.CheckTargetPath;
import org.caosdb.server.filesystem.VirtualFSODescriptorInterface;
import org.caosdb.server.jobs.FilesJob;
import org.caosdb.server.utils.EntityStatus;
import org.caosdb.server.utils.ServerMessages;

/**
 * Check whether the target path of a file entity is valid. That is, the target path is in the file
 * system and not used by any other file.
 *
 * @author tf
 */
public class CheckTargetPathValid extends FilesJob {

  @Override
  public final void run() {
    if (getEntity().hasFSODescriptor()) {
      final VirtualFSODescriptorInterface file = getEntity().getFSODescriptor();

      if (!file.hasPath()) {
        // the file doesn't have a path property at all
        getEntity().setEntityStatus(EntityStatus.UNQUALIFIED);
        getEntity().addError(ServerMessages.FILE_HAS_NO_TARGET_PATH);
        return;
      }

      // check that target path is not owned by another entity.
      final CheckTargetPath t = new CheckTargetPath(getEntity(), getContainer());
      execute(t);
    }
  }
}
