package org.caosdb.server.filesystem;

import static org.junit.jupiter.api.Assertions.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StringReader;
import org.apache.commons.io.input.ReaderInputStream;
import org.junit.jupiter.api.Test;

class HashTest {

  @Test
  void testSerializable() throws IOException, ClassNotFoundException {
    Hash hash = Hash.create("val", 2134L, "algo");
    ByteArrayOutputStream byteWriter = new ByteArrayOutputStream();
    ObjectOutputStream out = new ObjectOutputStream(byteWriter);
    out.writeObject(hash);
    out.close();
    byteWriter.close();
    byte[] serialized = byteWriter.toByteArray();
    ByteArrayInputStream byteReader = new ByteArrayInputStream(serialized);
    ObjectInputStream in = new ObjectInputStream(byteReader);
    Hash hash2 = (Hash) in.readObject();
    assertEquals(hash, hash2);
  }

  @Test
  void testSHA512Hasher() throws IOException {
    Hash hash =
        Hash.create(
            "e7c22b994c59d9cf2b48e549b1e24666636045930d3da7c1acb299d1c3b7f931f94aae41edda2c2b207a36e10f8bcb8d45223e54878f5b316e7ce3b6bc019629",
            0,
            Hasher.Default);
    StringReader reader = new StringReader("hello\n");
    ReaderInputStream in = new ReaderInputStream(reader);
    Hasher hasher = Hasher.create(hash.getAlgorithm());
    Hash hash2 = hasher.hash(in);
    assertEquals(hash, hash2);
  }

  @Test
  void testEquals() throws IOException {
    StringReader reader1 = new StringReader("hello\n");
    ReaderInputStream in1 = new ReaderInputStream(reader1);
    Hasher hasher1 = Hasher.create("sha-256");

    StringReader reader2 = new StringReader("hello\n");
    ReaderInputStream in2 = new ReaderInputStream(reader2);
    Hasher hasher2 = Hasher.create("SHA-256");

    assertEquals(hasher1.hash(in1), hasher2.hash(in2));
  }
}
