package org.caosdb.server.filesystem.consistency;

import static org.junit.jupiter.api.Assertions.assertEquals;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;
import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.caosdb.server.filesystem.consistency.ConsistencyConfig.LogLevel;
import org.junit.jupiter.api.Test;

public class ConsistencyConfigTest {

  private List<ConsistencyConfig> parse(String input) throws IOException {
    List<ConsistencyConfig> results = new LinkedList<>();
    ObjectMapper mapper = new ObjectMapper(new YAMLFactory());
    ObjectReader reader = mapper.readerFor(ConsistencyConfig.class);
    Iterator<ConsistencyConfig> configs = reader.readValues(new StringReader(input));
    configs.forEachRemaining(results::add);
    return results;
  }

  @Test
  public void testEmptyConfig() throws IOException {
    List<ConsistencyConfig> results = parse("");
    assertEquals(0, results.size());
  }

  @Test
  public void test1Config() throws IOException {
    List<ConsistencyConfig> results =
        parse(
            "eventsLogConfig:\n    FILE_UNKNOWN: ERROR\nautoUpdateConfig:\n    FILE_UNKNOWN: bla");

    assertEquals(1, results.size());
    assertEquals(LogLevel.ERROR, results.get(0).eventsLogConfig.get("FILE_UNKNOWN"));
    assertEquals(
        AutoUpdateAction.create("bla"), results.get(0).autoUpdateConfig.get("FILE_UNKNOWN"));
  }
}
